@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)files.c	3.1	92/12/07	*/
+/*	SCCS Id: @(#)files.c	3.1	93/06/27	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -17,7 +17,7 @@ extern int errno;
 #include <signal.h>
 #endif
 
-#if defined(MSDOS) || defined(OS2) || defined(TOS)
+#if defined(MSDOS) || defined(OS2) || defined(TOS) || defined(WIN32)
 # ifndef GNUDOS
 #include <sys\stat.h>
 # else
@@ -63,6 +63,8 @@ char SAVEP[SAVESIZE];	/* holds path of directory for save file */
 extern char PATH[];	/* see sys/amiga/amidos.c */
 #endif
 
+extern int n_dgns;		/* from dungeon.c */
+
 static char * FDECL(set_bonesfile_name, (char *,d_level*));
 
 /* fopen a file, with OS-dependent bells and whistles */
@@ -146,11 +148,11 @@ int lev;
 	 */
 	fd = open(lock, O_WRONLY |O_CREAT | O_TRUNC | O_BINARY, FCMASK);
 #else
-#ifdef MAC
-	fd = maccreat ( lock , LEVL_TYPE ) ;
-#else /* MAC */
+# ifdef MAC
+	fd = maccreat(lock, LEVL_TYPE);
+# else
 	fd = creat(lock, FCMASK);
-#endif /* MAC */
+# endif
 #endif /* MICRO */
 
 	return fd;
@@ -169,11 +171,11 @@ int lev;
 	if (fileinfo[lev].where != ACTIVE)
 		swapin_file(lev);
 #endif
-# ifdef MAC
-	fd = macopen ( lock , O_RDONLY | O_BINARY , LEVL_TYPE ) ;
-# else /* MAC */
+#ifdef MAC
+	fd = macopen(lock, O_RDONLY | O_BINARY, LEVL_TYPE);
+#else
 	fd = open(lock, O_RDONLY | O_BINARY, 0);
-# endif /* MAC */
+#endif
 	return fd;
 }
 
@@ -202,7 +204,8 @@ clearlocks()
 # if defined(UNIX) || defined(VMS)
 	(void) signal(SIGHUP, SIG_IGN);
 # endif
-	for (x = maxledgerno(); x >= 0; x--)
+	/* can't access maxledgerno() before dungeons are created -dlc */
+	for (x = (n_dgns ? maxledgerno() : 0); x >= 0; x--)
 		delete_levelfile(x);	/* not all levels need be present */
 #endif
 }
@@ -253,11 +256,11 @@ char **bonesid;
 	fd = open(bones, O_WRONLY |O_CREAT | O_TRUNC | O_BINARY, FCMASK);
 #else
 # ifdef MAC
-	fd = maccreat ( bones , BONE_TYPE ) ;
-# else /* MAC */
+	fd = maccreat(bones, BONE_TYPE);
+# else
 	fd = creat(bones, FCMASK);
-# endif /* MAC */
-#  if defined(VMS) && !defined(SECURE)
+# endif
+# if defined(VMS) && !defined(SECURE)
 	/*
 	   Re-protect bones file with world:read+write+execute+delete access.
 	   umask() doesn't seem very reliable; also, vaxcrtl won't let us set
@@ -267,7 +270,7 @@ char **bonesid;
 	   denies some or all access to world.
 	 */
 	(void) chmod(bones, FCMASK | 007);  /* allow other users full access */
-#  endif /* VMS && !SECURE */
+# endif /* VMS && !SECURE */
 #endif /* MICRO */
 
 	return fd;
@@ -284,7 +287,7 @@ char **bonesid;
 	*bonesid = set_bonesfile_name(bones, lev);
 	uncompress(bones);	/* no effect if nonexistent */
 #ifdef MAC
-	fd = macopen ( bones , O_RDONLY | O_BINARY , BONE_TYPE ) ;
+	fd = macopen(bones, O_RDONLY | O_BINARY, BONE_TYPE);
 #else
 	fd = open(bones, O_RDONLY | O_BINARY, 0);
 #endif
@@ -340,8 +343,8 @@ set_savefile_name()
 # else
 	Sprintf(SAVEF, "save/%d%s", (int)getuid(), plname);
 	regularize(SAVEF+5);	/* avoid . or / in name */
-# endif
-#endif
+# endif	/* MICRO */
+#endif	/* VMS */
 }
 
 #ifdef INSURANCE
@@ -366,11 +369,11 @@ set_error_savefile()
       }
 	Strcat(SAVEF, ".e;1");
 # else
-#ifdef MAC
+#  ifdef MAC
 	Strcat(SAVEF, "-e");
-#else
+#  else
 	Strcat(SAVEF, ".e");
-#endif
+#  endif
 # endif
 }
 #endif
@@ -381,30 +384,30 @@ int
 create_savefile()
 {
 	int fd;
-# ifdef AMIGA
+#ifdef AMIGA
 	fd = ami_wbench_getsave(O_WRONLY | O_CREAT | O_TRUNC);
-# else
-#  ifdef MICRO
+#else
+# ifdef MICRO
 	fd = open(SAVEF, O_WRONLY | O_BINARY | O_CREAT | O_TRUNC, FCMASK);
+# else
+#  ifdef MAC
+	fd = maccreat(SAVEF, SAVE_TYPE);
 #  else
-#   ifdef MAC
-	fd = creat(SAVEF, SAVE_TYPE);
-#   else /* MAC */
 	fd = creat(SAVEF, FCMASK);
-#   endif /* MAC */
-#   if defined(VMS) && !defined(SECURE)
+#  endif
+#  if defined(VMS) && !defined(SECURE)
 	/*
 	   Make sure the save file is owned by the current process.  That's
 	   the default for non-privileged users, but for priv'd users the
 	   file will be owned by the directory's owner instead of the user.
 	 */
-#    ifdef getuid	/*(see vmsunix.c)*/
-#     undef getuid
-#    endif
+#   ifdef getuid	/*(see vmsunix.c)*/
+#    undef getuid
+#   endif
 	(void) chown(SAVEF, getuid(), getgid());
-#   endif /* VMS && !SECURE */
-#  endif
-# endif
+#  endif /* VMS && !SECURE */
+# endif	/* MICRO */
+#endif	/* AMIGA */
 
 	return fd;
 }
@@ -420,10 +423,10 @@ open_savefile()
 	fd = ami_wbench_getsave(O_RDONLY);
 #else
 # ifdef MAC
-	fd = macopen ( SAVEF , O_RDONLY , SAVE_TYPE ) ;
-# else /* MAC */
-	fd = open(SAVEF, O_RDONLY, 0);
-# endif /* MAC */
+	fd = macopen(SAVEF, O_RDONLY | O_BINARY, SAVE_TYPE);
+# else
+	fd = open(SAVEF, O_RDONLY | O_BINARY, 0);
+# endif
 #endif /* AMIGA */
 	return fd;
 }
@@ -446,22 +449,86 @@ delete_savefile()
 
 /* ----------  BEGIN FILE COMPRESSION HANDLING ----------- */
 
+#ifdef COMPRESS
+/* 
+ * using system() is simpler, but opens up security holes and causes
+ * problems on at least Interactive UNIX 3.0.1 (SVR3.2), where any
+ * setuid is renounced by /bin/sh, so the files cannot be accessed.
+ *
+ * cf. child() in unixunix.c.
+ */
+void
+docompress_file(filename, uncomp)
+char *filename;
+boolean uncomp;
+{
+	char *args[10];
+# ifdef COMPRESS_OPTIONS
+	char opts[80];
+# endif
+	int i = 0;
+	int f;
+
+	args[0] = COMPRESS;
+	if (uncomp) args[++i] = "-d";	/* uncompress */
+# ifdef COMPRESS_OPTIONS
+	{
+	    /* we can't guarantee there's only one additional option, sigh */
+	    char *opt;
+	    boolean inword = FALSE;
+
+	    Strcpy(opts, COMPRESS_OPTIONS);
+	    opt = opts;
+	    while (*opt) {
+		if ((*opt == ' ') || (*opt == '\t')) {
+		    if (inword) {
+			*opt = '\0';
+			inword = FALSE;
+		    }
+		} else if (!inword) {
+		    args[++i] = opt;
+		    inword = TRUE;
+		}
+		opt++;
+	    }
+	}
+# endif
+	args[++i] = filename;
+	args[++i] = NULL;
+
+	f = fork();
+	if (f == 0) {	/* child */
+		(void) execv(args[0], args);
+		perror(NULL);
+		pline("Exec to %scompress %s failed.",
+			uncomp ? "un" : "", filename);
+		exit(1);
+	} else if (f == -1) {
+		perror(NULL);
+		pline("Fork to %scompress %s failed.",
+			uncomp ? "un" : "", filename);
+		return;
+	}
+	(void) signal(SIGINT, SIG_IGN);
+	(void) signal(SIGQUIT, SIG_IGN);
+	(void) wait((int *)0);
+	(void) signal(SIGINT, (SIG_RET_TYPE) done1);
+# ifdef WIZARD
+	if (wizard) (void) signal(SIGQUIT, SIG_DFL);
+# endif
+}
+#endif
+
 /* compress file */
 void
 compress(filename)
 const char *filename;
+#ifdef applec
+# pragma unused(filename)
+#endif
 {
 #ifdef COMPRESS
-	char cmd[80];
-
-	Strcpy(cmd, COMPRESS);
-	Strcat(cmd, " ");
-# ifdef COMPRESS_OPTIONS
-	Strcat(cmd, COMPRESS_OPTIONS);
-	Strcat(cmd, " ");
-# endif
-	Strcat(cmd,filename);
-	(void) system(cmd);
+	docompress_file(filename, FALSE);
 #endif
 }
 
@@ -470,23 +537,21 @@ const char *filename;
 void
 uncompress(filename)
 const char *filename;
+#ifdef applec
+# pragma unused(filename)
+#endif
 {
 #ifdef COMPRESS
-	char cmd[80], cfn[80];
+	char cfn[80];
 	int fd;
 
-	Strcpy(cfn,filename);
-	Strcat(cfn,".Z");
-	if((fd = open(cfn,O_RDONLY)) >= 0) {
-	    (void) close(fd);
-	    Strcpy(cmd, COMPRESS);
-	    Strcat(cmd, " -d ");        /* uncompress */
-# ifdef COMPRESS_OPTIONS
-	    Strcat(cmd, COMPRESS_OPTIONS);
-	    Strcat(cmd, " ");
+	Strcpy(cfn, filename);
+# ifdef COMPRESS_EXTENSION
+	Strcat(cfn, COMPRESS_EXTENSION);
 # endif
-	    Strcat(cmd,cfn);
-	    (void) system(cmd);
+	if ((fd = open(cfn, O_RDONLY)) >= 0) {
+		(void) close(fd);
+		docompress_file(cfn, TRUE);
 	}
 #endif
 }
@@ -535,6 +600,9 @@ boolean
 lock_file(filename, retryct)
 const char *filename;
 int retryct;
+#ifdef applec
+# pragma unused(filename, retryct)
+#endif
 {
 #if defined(UNIX) || defined(VMS)
 	char *lockname;
@@ -556,6 +624,14 @@ int retryct;
 			HUP raw_printf("No write permission to lock %s!",
 				       filename);
 			return FALSE;
+# ifdef VMS			/* c__translate(vmsfiles.c) */
+		    case EPERM:
+			/* could be misleading, but usually right */
+			HUP raw_printf(
+				  "Can't lock %s due to directory protection.",
+				       filename);
+			return FALSE;
+# endif
 		    case EEXIST:
 			break;	/* retry checks below */
 		    default:
@@ -596,6 +672,9 @@ int retryct;
 void
 unlock_file(filename)
 const char *filename;
+#if defined(applec)
+# pragma unused(filename)
+#endif
 {
 #if defined(UNIX) || defined(VMS)
 	char *lockname;
@@ -620,11 +699,11 @@ const char *configfile =
 #ifdef UNIX
 			".nethackrc";
 #else
-#ifdef MAC
+# ifdef MAC
 			"NetHack defaults";
-#else
+# else
 			"NetHack.cnf";
-#endif
+# endif
 #endif
 
 static FILE *FDECL(fopen_config_file, (const char *));
@@ -645,9 +724,25 @@ const char *filename;
 #endif
 
 	/* "filename" is an environment variable, so it should hang around */
-	if (filename && ((fp = fopenp(filename, "r")) != (FILE *)0)) {
-		configfile = filename;
-		return(fp);
+	if (filename) {
+#ifdef UNIX
+		if (access(filename, 4) == -1) {
+			/* 4 is R_OK on newer systems */
+			/* nasty sneaky attempt to read file through
+			 * NetHack's setuid permissions -- this is the only
+			 * place a file name may be wholly under the player's
+			 * control
+			 */
+			raw_printf("Access to %s denied (%d).",
+					filename, errno);
+			wait_synch();
+			/* fall through to standard names */
+		} else
+#endif
+		if ((fp = fopenp(filename, "r")) != (FILE *)0) {
+			configfile = filename;
+			return(fp);
+		}
 	}
 
 #if defined(MICRO) || defined(MAC)
@@ -739,6 +834,9 @@ FILE		*fp;
 char		*buf;
 char		*tmp_ramdisk;
 char		*tmp_levels;
+#if defined(applec)
+# pragma unused(tmp_ramdisk,tmp_levels)
+#endif
 {
 	char		*bufp, *altp;
 
@@ -785,16 +883,23 @@ char		*tmp_levels;
 		(void) strncpy(tmp_levels, bufp, PATHLEN);
 
 	} else if (!strncmpi(buf, "SAVE", 4)) {
+# ifdef MFLOPPY
+		extern	int saveprompt;
+#endif
 		char *ptr;
 		if (ptr = index(bufp, ';')) {
 			*ptr = '\0';
 # ifdef MFLOPPY
 			if (*(ptr+1) == 'n' || *(ptr+1) == 'N') {
-				extern	int saveprompt;
 				saveprompt = FALSE;
 			}
 # endif
-    }
+		}
+#ifdef	MFLOPPY
+		else
+		    saveprompt = flags.asksavedisk;
+#endif
+
 		(void) strncpy(SAVEP, bufp, PATHLEN);
 		append_slash(SAVEP);
 #endif /* MICRO */
@@ -822,19 +927,48 @@ char		*tmp_levels;
 	    /* monsyms[0] is unused */
 	    (void) get_uchars(fp, buf, bufp, &(monsyms[1]),
 					MAXMCLASSES-1, "MONSTERS");
+#ifdef VIDEOSHADES
+	} else if (!strncmpi(buf, "VIDEOCOLORS", 6)) {
+	    uchar   transcolors[MAXCOLORS];
+	    int   len;
+
+	    len = get_uchars(fp, buf, bufp, transcolors,
+					MAXCOLORS, "VIDEOCOLORS");
+	    assign_videocolors(transcolors, len);
+	} else if (!strncmpi(buf, "VIDEOSHADES", 6)) {
+	    int   len;
+
+	    len = strlen(bufp);
+	    assign_videoshades(bufp, len);
+#endif
 #ifdef AMIGA
+	} else if (!strncmpi(buf, "FONT", 4)) {
+		char *t;
+		extern void amii_set_text_font( char *, int );
+
+		if( t = strchr( buf+5, ':' ) )
+		{
+		    *t = 0;
+		    amii_set_text_font( buf+5, atoi( t + 1 ) );
+		    *t = ':';
+		}
 	} else if (!strncmpi(buf, "PATH", 4)) {
 		(void) strncpy(PATH, bufp, PATHLEN);
+#endif
+#ifdef AMIGA
 	} else if (!strncmpi(buf, "PENS", 3)) {
+# ifdef AMII_GRAPHICS
 		int i;
 		char *t;
-		extern unsigned short amii_curmap[];
-		for (i = 0, t = strtok(bufp, ",");
-				t && i < 8;
-				t = strtok(NULL, ","), ++i) {
-			sscanf(t, "%hx", &amii_curmap[i]);
+		extern void amii_setpens( void );
+
+		for (i = 0, t = strtok(bufp, ",/"); t != NULL;
+				    t = strtok(NULL, ",/"), ++i)
+		{
+			sscanf(t, "%hx", &flags.amii_curmap[i]);
 		}
 		amii_setpens();
+# endif
 #endif
 	} else
 		return 0;
@@ -861,28 +995,26 @@ const char *filename;
 	char	buf[BUFSZ];
 	FILE	*fp;
 
-#if defined(MAC)
-	{
-		long nul = 0L ;
-		Str255 volName ;
-		/*
-		 * We should try and get this data from a rsrc, in the profile file
-		 * the user double-clicked... This data should be saved with the
-		 * save file in the resource fork, AND be saveable in "stationery"
-		 */
-		GetVol ( volName , & theDirs . dataRefNum ) ;
-		GetWDInfo ( theDirs . dataRefNum , & theDirs . dataRefNum , & theDirs .
-			dataDirID , & nul ) ;
-		if ( volName [ 0 ] > 31 ) volName [ 0 ] = 31 ;
-		for ( nul = 1 ; nul <= volName [ 0 ] ; nul ++ ) {
-			if ( volName [ nul ] == ':' ) {
-				volName [ nul ] = 0 ;
-				volName [ 0 ] = nul - 1 ;
-				break ;
-			}
+#ifdef MAC
+	long nul = 0L ;
+	Str255 volName ;
+	/*
+	 * We should try to get this data from a rsrc, in the profile file
+	 * the user double-clicked...  This data should be saved with the
+	 * save file in the resource fork, AND be saveable in "stationery"
+	 */
+	GetVol ( volName , & theDirs . dataRefNum ) ;
+	GetWDInfo ( theDirs . dataRefNum , & theDirs . dataRefNum , & theDirs .
+		dataDirID , & nul ) ;
+	if ( volName [ 0 ] > 31 ) volName [ 0 ] = 31 ;
+	for ( nul = 1 ; nul <= volName [ 0 ] ; nul ++ ) {
+		if ( volName [ nul ] == ':' ) {
+			volName [ nul ] = 0 ;
+			volName [ 0 ] = nul - 1 ;
+			break ;
 		}
-		BlockMove ( volName , theDirs . dataName , 32L ) ;
 	}
+	BlockMove ( volName , theDirs . dataName , 32L ) ;
 #endif /* MAC */
 
 	if (!(fp = fopen_config_file(filename))) return;
@@ -930,6 +1062,9 @@ const char *filename;
 void
 check_recordfile(dir)
 const char *dir;
+#if defined(applec)
+# pragma unused(dir)
+#endif
 {
 #if defined(UNIX) || defined(VMS)
 	int fd = open(RECORD, O_RDWR, 0);
